import multiprocessing

import zmq


class Worker(multiprocessing.Process):
    """Worker that receives data from and sends results to an mpzmq proxy.

    Attributes:
        backend: A string that contains the location and port of the mpzmq
            worker-side proxy.
            Defaults to "127.0.0.1:5560".
    """
    def __init__(self, backend="127.0.0.1:5560"):
        """Inits Worker with connection settings and runs user-defined init."""
        super(Worker, self).__init__()
        self.backend = backend
        self.init()

    def run(self):
        """Connects to ZeroMQ proxy and runs user-defined work."""
        context = zmq.Context()
        socket = context.socket(zmq.REP)
        socket.connect("tcp://{}".format(self.backend))
        while 1:
            data = socket.recv()
            result = self.work(data)
            if isinstance(result, str):
                result = result.encode()
            socket.send(result)

    def init(self):
        """User-defined process initialization."""
        pass

    def work(self, data):
        """User-defined data processing."""
        return b""


class Proxy(multiprocessing.Process):
    """Proxy that connects mpzmq clients to mpzmq workers.

    Attributes:
        frontend_port: An integer that contains the port that the proxy should
            listen for client connections on.
            Defaults to 5559.
        backend_port: An integer that contains the port that the proxy should
            listen for worker connections on.
            Defaults to 5560.
    """
    def __init__(self, frontend_port=5559, backend_port=5560):
        """Inits Proxy with connection settings."""
        super(Proxy, self).__init__()
        self.frontend_port = frontend_port
        self.backend_port = backend_port

    def run(self):
        """Runs ZeroMQ proxy for connections between clients and workers."""
        context = zmq.Context()
        frontend = context.socket(zmq.XREP)
        frontend.bind("tcp://*:{}".format(self.frontend_port))
        backend = context.socket(zmq.XREQ)
        backend.bind("tcp://*:{}".format(self.backend_port))
        zmq.proxy(frontend, backend)



import zmq


class Client(object):
    """Client that sends data to and receives results from an mpzmq proxy.

    Attributes:
        frontend: A string that contains the location and port of the mpzmq
            client-side proxy.
            Defaults to "127.0.0.1:5559".
        recv_timeout: An integer that controls how long (in milliseconds) to wait
            for a response from the mpzmq server.
            Defaults to -1 (waits forever).
    """
    def __init__(self, frontend="127.0.0.1:5559", recv_timeout=-1):
        """Inits Client with connection and polling settings."""
        self.frontend = frontend
        self.recv_timeout = recv_timeout

    def create_zmq(self):
        """Creates ZeroMQ connection to proxy."""
        self.context = zmq.Context()
        self.socket = self.context.socket(zmq.REQ)
        self.socket.setsockopt(zmq.LINGER, 0)
        self.socket.setsockopt(zmq.RCVTIMEO, self.recv_timeout)
        self.socket.setsockopt(zmq.REQ_CORRELATE, 1)
        self.socket.setsockopt(zmq.REQ_RELAXED, 1)
        self.socket.connect("tcp://{}".format(self.frontend))

    def destroy_zmq(self):
        """Shuts down ZeroMQ connection to proxy."""
        self.context.destroy()

    def request(self, data):
        """Sends data to and receives result from proxy."""
        if isinstance(data, str):
            data = data.encode()
        self.socket.send(data)

        try:
            return self.socket.recv()
        except zmq.error.Again:
            return None



from setuptools import setup, find_packages

setup(
    name='mpzmq',
    packages=['mpzmq'],
    version='0.1',
    license='MIT',
    author='Josh Liburdi @jshlbrd',
    author_email='liburdi.joshua@gmail.com',
    description='minimalist library for building scalable, distributed python applications',
    long_description='This is a minimalist library for building scalable, distributed Python applications using multiprocessing and ZeroMQ (ZMQ). See project homepage for more details.',
    url='https://github.com/jshlbrd/mpzmq',
    keywords=[
        'zmq',
        'pyzmq',
        'mp',
        'multiprocessing',
        'distributed'
    ],
    install_requires=[
        'pyzmq'
    ],
    classifiers=[
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
    ]
)
